@@ -6,13 +6,9 @@
 import re
 
 from git.cmd import handle_process_output
-from git.compat import (
-    defenc,
-    PY3
-)
+from git.compat import defenc
 from git.util import finalize_process, hex_to_bin
 
-from .compat import binary_type
 from .objects.blob import Blob
 from .objects.util import mode_str_to_int
 
@@ -28,10 +24,7 @@ _octal_byte_re = re.compile(b'\\\\([0-9]{3})')
 def _octal_repl(matchobj):
     value = matchobj.group(1)
     value = int(value, 8)
-    if PY3:
-        value = bytes(bytearray((value,)))
-    else:
-        value = chr(value)
+    value = bytes(bytearray((value,)))
     return value
 
 
@@ -115,6 +108,7 @@ class Diffable(object):
             args.append("-p")
         else:
             args.append("--raw")
+            args.append("-z")
 
         # in any way, assure we don't see colored output,
         # fixes https://github.com/gitpython-developers/GitPython/issues/172
@@ -268,8 +262,8 @@ class Diff(object):
         self.a_mode = a_mode
         self.b_mode = b_mode
 
-        assert a_rawpath is None or isinstance(a_rawpath, binary_type)
-        assert b_rawpath is None or isinstance(b_rawpath, binary_type)
+        assert a_rawpath is None or isinstance(a_rawpath, bytes)
+        assert b_rawpath is None or isinstance(b_rawpath, bytes)
         self.a_rawpath = a_rawpath
         self.b_rawpath = b_rawpath
 
@@ -282,7 +276,7 @@ class Diff(object):
         # we need to overwrite "repo" to the corresponding submodule's repo instead
         if repo and a_rawpath:
             for submodule in repo.submodules:
-                if submodule.path == a_rawpath.decode("utf-8"):
+                if submodule.path == a_rawpath.decode(defenc, 'replace'):
                     if submodule.module_exists():
                         repo = submodule.module()
                     break
@@ -302,8 +296,8 @@ class Diff(object):
         self.copied_file = copied_file
 
         # be clear and use None instead of empty strings
-        assert raw_rename_from is None or isinstance(raw_rename_from, binary_type)
-        assert raw_rename_to is None or isinstance(raw_rename_to, binary_type)
+        assert raw_rename_from is None or isinstance(raw_rename_from, bytes)
+        assert raw_rename_to is None or isinstance(raw_rename_to, bytes)
         self.raw_rename_from = raw_rename_from or None
         self.raw_rename_to = raw_rename_to or None
 
@@ -370,8 +364,6 @@ class Diff(object):
         # Python2 silliness: have to assure we convert our likely to be unicode object to a string with the
         # right encoding. Otherwise it tries to convert it using ascii, which may fail ungracefully
         res = h + msg
-        if not PY3:
-            res = res.encode(defenc)
         # end
         return res
 
@@ -487,55 +479,55 @@ class Diff(object):
 
         index = DiffIndex()
 
-        def handle_diff_line(line):
-            line = line.decode(defenc)
-            if not line.startswith(":"):
-                return
+        def handle_diff_line(lines):
+            lines = lines.decode(defenc)
 
-            meta, _, path = line[1:].partition('\t')
-            old_mode, new_mode, a_blob_id, b_blob_id, _change_type = meta.split(None, 4)
-            # Change type can be R100
-            # R: status letter
-            # 100: score (in case of copy and rename)
-            change_type = _change_type[0]
-            score_str = ''.join(_change_type[1:])
-            score = int(score_str) if score_str.isdigit() else None
-            path = path.strip()
-            a_path = path.encode(defenc)
-            b_path = path.encode(defenc)
-            deleted_file = False
-            new_file = False
-            copied_file = False
-            rename_from = None
-            rename_to = None
+            for line in lines.split(':')[1:]:
+                meta, _, path = line.partition('\x00')
+                path = path.rstrip('\x00')
+                old_mode, new_mode, a_blob_id, b_blob_id, _change_type = meta.split(None, 4)
+                # Change type can be R100
+                # R: status letter
+                # 100: score (in case of copy and rename)
+                change_type = _change_type[0]
+                score_str = ''.join(_change_type[1:])
+                score = int(score_str) if score_str.isdigit() else None
+                path = path.strip()
+                a_path = path.encode(defenc)
+                b_path = path.encode(defenc)
+                deleted_file = False
+                new_file = False
+                copied_file = False
+                rename_from = None
+                rename_to = None
 
-            # NOTE: We cannot conclude from the existence of a blob to change type
-            # as diffs with the working do not have blobs yet
-            if change_type == 'D':
-                b_blob_id = None
-                deleted_file = True
-            elif change_type == 'A':
-                a_blob_id = None
-                new_file = True
-            elif change_type == 'C':
-                copied_file = True
-                a_path, b_path = path.split('\t', 1)
-                a_path = a_path.encode(defenc)
-                b_path = b_path.encode(defenc)
-            elif change_type == 'R':
-                a_path, b_path = path.split('\t', 1)
-                a_path = a_path.encode(defenc)
-                b_path = b_path.encode(defenc)
-                rename_from, rename_to = a_path, b_path
-            elif change_type == 'T':
-                # Nothing to do
-                pass
-            # END add/remove handling
+                # NOTE: We cannot conclude from the existence of a blob to change type
+                # as diffs with the working do not have blobs yet
+                if change_type == 'D':
+                    b_blob_id = None
+                    deleted_file = True
+                elif change_type == 'A':
+                    a_blob_id = None
+                    new_file = True
+                elif change_type == 'C':
+                    copied_file = True
+                    a_path, b_path = path.split('\x00', 1)
+                    a_path = a_path.encode(defenc)
+                    b_path = b_path.encode(defenc)
+                elif change_type == 'R':
+                    a_path, b_path = path.split('\x00', 1)
+                    a_path = a_path.encode(defenc)
+                    b_path = b_path.encode(defenc)
+                    rename_from, rename_to = a_path, b_path
+                elif change_type == 'T':
+                    # Nothing to do
+                    pass
+                # END add/remove handling
 
-            diff = Diff(repo, a_path, b_path, a_blob_id, b_blob_id, old_mode, new_mode,
-                        new_file, deleted_file, copied_file, rename_from, rename_to,
-                        '', change_type, score)
-            index.append(diff)
+                diff = Diff(repo, a_path, b_path, a_blob_id, b_blob_id, old_mode, new_mode,
+                            new_file, deleted_file, copied_file, rename_from, rename_to,
+                            '', change_type, score)
+                index.append(diff)
 
         handle_process_output(proc, handle_diff_line, None, finalize_process, decode_streams=False)
 
